// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/dot/siena.html
//
//  Author: Antonio Carzaniga <carzanig@cs.colorado.edu>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 1998-1999 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: object.cc,v 1.15 2002/12/02 17:36:51 carzanig Exp $
//
#include <cstdio>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <string>
#include <iostream>
#include <sstream>
#include <exception>

#include "siena.h"
#include "siena/Monitor.h"

//
// quick hack here!
//
#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN	256
#endif

Request			PubRequest;
Request			SubRequest;

TCPPort *		Port = NULL;

URI *			Uri = NULL;
string			UriStr;
string			MyUriStr;

SimpleMonitor *		SienaMonitor = NULL;

GtkWidget *		TextWindow;
GtkWidget *		LogText;
GtkWidget *		SienaText;
GtkWidget *		StatusBar;

GtkWidget *		SienaObject;

GtkWidget *		ConfirmQuitDialog;
GtkWidget *		AboutDialog;
GtkFileSelection *	FileSelection;

GdkFont *		LogFont;

gchar * LogFontName = "-adobe-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1";

GdkColor ErrorColor;
GdkColor EventColor;
GdkColor ReplyColor;
GdkColor BackgroundColor;

static int init_fonts() 
{
    LogFont = gdk_font_load(LogFontName);
    if (LogFont == NULL) {
	g_error("couldn't load font %s", LogFontName);
	return 1;
    } else {
	return 0;
    }
}

static void init_colors()
{
    GdkColormap * cmap;

    cmap = gdk_colormap_get_system();

    ErrorColor.red = 0xffff;
    ErrorColor.green = 0x0;
    ErrorColor.blue = 0x0;
    if (!gdk_color_alloc(cmap, &ErrorColor)) {
	g_error("couldn't allocate Errors color");
    }

    ReplyColor.red = 0x0;
    ReplyColor.green = 0x0;
    ReplyColor.blue = 0x0;
    if (!gdk_color_alloc(cmap, &ReplyColor)) {
	g_error("couldn't allocate Reply color");
    }

    EventColor.red = 0x0;
    EventColor.green = 0x0;
    EventColor.blue = 0xffff;
    if (!gdk_color_alloc(cmap, &EventColor)) {
	g_error("couldn't allocate Events color");
    }
    
    BackgroundColor.red = 0xffff;
    BackgroundColor.green = 0xffff;
    BackgroundColor.blue = 0xffff;
    if (!gdk_color_alloc(cmap, &BackgroundColor)) {
	g_error("couldn't allocate Background color");
    }
}

static void new_clicked(GtkButton *button, gpointer user_data)
{
    gtk_editable_delete_text(GTK_EDITABLE(LogText), 0,
			     gtk_text_get_length(GTK_TEXT(LogText)));
}

static void quit_clicked(GtkButton *button, gpointer user_data)
{
    if (SienaMonitor != NULL) {
	SienaMonitor->remove_node(MyUriStr);
    }
    gtk_main_quit();
}

static void publish_clicked(GtkButton *button, gpointer user_data)
{   
    GtkText * text = GTK_TEXT(SienaText);

    gint length = gtk_text_get_length(text);
    const char * s = gtk_editable_get_chars(GTK_EDITABLE(text), 0, length);

    try {
	Event e;

	istringstream ist(s);
	ist >> e;
	if (SienaMonitor != NULL) {
	    SienaMonitor->notify(MyUriStr, UriStr);
	}
    
	iostream * o = Uri->open_connection();
	*o << PubRequest << e << endl<< flush;
	char reply[1024];
	o->getline(reply, 1024);
	gtk_text_insert(GTK_TEXT(LogText), LogFont,
			&ReplyColor, &BackgroundColor, reply, strlen(reply));
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, "\n", 1);
	delete(o);
    } catch (exception &ex) {
	gtk_text_insert(GTK_TEXT(LogText), LogFont,
			&ErrorColor, &BackgroundColor,
			ex.what(), strlen(ex.what()));
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, "\n", 1);
    }
}

static void subscribe_clicked(GtkButton *button, gpointer user_data)
{   
    const string & method = *((const string *)user_data);
    SubRequest["method"] = method;

    GtkText * text = GTK_TEXT(SienaText);

    gint length = gtk_text_get_length(text);
    const char * s = gtk_editable_get_chars(GTK_EDITABLE(text), 0, length);

    try {
	Filter f;

	istringstream ist(s);
	ist >> f;
    
	if (SienaMonitor != NULL) {
	    if (SENP::SUB == method) {
		SienaMonitor->subscribe(MyUriStr, UriStr);
	    } else if (SENP::UNS == method) {
		SienaMonitor->unsubscribe(MyUriStr, UriStr);
	    }
	}

	iostream * o = Uri->open_connection();
	*o << SubRequest << f << endl<< flush;
	char reply[1024];
	o->getline(reply, 1024);
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ReplyColor, &BackgroundColor, reply, strlen(reply));
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, "\n", 1);

	delete(o);
    } catch (exception &ex) {
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, 
			ex.what(), strlen(ex.what()));
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, "\n", 1);
    }
}

static void open_dialog(GtkMenuItem *menuitem, gpointer user_data)
{
    gtk_widget_show(GTK_WIDGET(user_data));
}

static void close_dialog(GtkMenuItem *menuitem, gpointer user_data)
{
    gtk_widget_hide(GTK_WIDGET(user_data));
}

static void create_AboutDialog ()
{
    GtkWidget *dialog_vbox1;
    GtkWidget *frame;
    GtkWidget *label;
    GtkWidget *dialog_action_area1;
    GtkWidget *close;

    AboutDialog = gtk_dialog_new ();
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "AboutDialog", AboutDialog);
    gtk_window_set_title (GTK_WINDOW (AboutDialog), "Siena Monitor - Information");
    gtk_window_set_policy (GTK_WINDOW (AboutDialog), TRUE, TRUE, FALSE);

    dialog_vbox1 = GTK_DIALOG (AboutDialog)->vbox;
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "dialog_vbox1", dialog_vbox1);
    gtk_widget_show (dialog_vbox1);

    frame = gtk_frame_new ("Siena Monitor");
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "frame", frame);
    gtk_widget_show (frame);
    gtk_box_pack_start (GTK_BOX (dialog_vbox1), frame, TRUE, TRUE, 0);

    label = gtk_label_new ("Siena:\nInternet-scale Content-based Event Notification\n\nSoftware Engineering Research Laboratory\nUniversity of Colorado\n\nContact:\nAntonio Carzaniga <carzanig@cs.colorado.edu>\nhttp://www.cs.colorado.edu/~serl/dot/siena.html");
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "label", label);
    gtk_widget_show (label);
    gtk_container_add (GTK_CONTAINER (frame), label);

    dialog_action_area1 = GTK_DIALOG (AboutDialog)->action_area;
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "dialog_action_area1", dialog_action_area1);
    gtk_widget_show (dialog_action_area1);
    gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

    close = gtk_button_new_with_label ("Close");
    gtk_object_set_data (GTK_OBJECT (AboutDialog), "close", close);
    gtk_widget_show (close);
    gtk_box_pack_start (GTK_BOX (dialog_action_area1), close, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (close), "clicked",
			GTK_SIGNAL_FUNC (close_dialog),
			AboutDialog);
}

static void create_ConfirmQuitDialog()
{
    GtkWidget *dialog_vbox;
    GtkWidget *label1;
    GtkWidget *dialog_action_area1;
    GtkWidget *hbox1;
    GtkWidget *OkQuitButton;
    GtkWidget *CancelQuitButton;

    ConfirmQuitDialog = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (ConfirmQuitDialog), "Confirm quit?");
    gtk_window_set_policy (GTK_WINDOW (ConfirmQuitDialog), TRUE, TRUE, FALSE);

    dialog_vbox = GTK_DIALOG (ConfirmQuitDialog)->vbox;
    gtk_widget_show (dialog_vbox);

    label1 = gtk_label_new ("Are you sure\nyou want to quit?\n");
    gtk_widget_show (label1);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), label1, TRUE, TRUE, 0);

    dialog_action_area1 = GTK_DIALOG (ConfirmQuitDialog)->action_area;
    gtk_widget_show (dialog_action_area1);
    gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 5);

    hbox1 = gtk_hbox_new (FALSE, 10);
    gtk_widget_show (hbox1);
    gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox1, TRUE, TRUE, 0);

    OkQuitButton = gtk_button_new_with_label ("Quit");
    gtk_widget_show (OkQuitButton);
    gtk_box_pack_start (GTK_BOX (hbox1), OkQuitButton, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (OkQuitButton), "clicked",
			GTK_SIGNAL_FUNC (quit_clicked),
			NULL);

    CancelQuitButton = gtk_button_new_with_label ("Cancel");
    gtk_widget_show (CancelQuitButton);
    gtk_box_pack_start (GTK_BOX (hbox1), CancelQuitButton, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (CancelQuitButton), "clicked",
			GTK_SIGNAL_FUNC (close_dialog),
			ConfirmQuitDialog);
}

static void create_SienaObject(const char * uri)
{
    char wname[1024];

    GtkWidget *vbox1;
    GtkWidget *menubar;
    GtkWidget *file;
    GtkWidget *file_menu;
    GtkWidget *new_b;
    GtkWidget *quit;
    GtkWidget *help;
    GtkWidget *help_menu;
    GtkWidget *about;
    GtkWidget *request_frame;
    GtkWidget *hbox2;
    GtkWidget *vbox2;
    GtkWidget *publish;
    GtkWidget *hbox3;
    GtkWidget *subscribe;
    GtkWidget *unsubscribe;
    GtkWidget *hbox4;
    GtkWidget *advertise;
    GtkWidget *unadvertise;
    GtkWidget *hbox1;
    GtkWidget *scrollbar;
    GtkStyle * sty;

    SienaObject = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "SienaObject", SienaObject);
    sprintf(wname, "Siena Object - %s", uri);
    gtk_window_set_title (GTK_WINDOW (SienaObject), wname);
    gtk_window_set_policy (GTK_WINDOW (SienaObject), TRUE, TRUE, FALSE);

    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "vbox1", vbox1);
    gtk_widget_show (vbox1);
    gtk_container_add (GTK_CONTAINER (SienaObject), vbox1);

    menubar = gtk_menu_bar_new ();
    gtk_object_set_data (GTK_OBJECT (SienaObject), "menubar", menubar);
    gtk_widget_show (menubar);
    gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, TRUE, 0);

    file = gtk_menu_item_new_with_label ("File");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "file", file);
    gtk_widget_show (file);
    gtk_container_add (GTK_CONTAINER (menubar), file);

    file_menu = gtk_menu_new ();
    gtk_object_set_data (GTK_OBJECT (SienaObject), "file_menu", file_menu);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (file), file_menu);

    new_b = gtk_menu_item_new_with_label("New");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "new", new_b);
    gtk_widget_show (new_b);
    gtk_container_add (GTK_CONTAINER (file_menu), new_b);
    gtk_signal_connect (GTK_OBJECT (new_b), "activate",
			GTK_SIGNAL_FUNC (new_clicked),
			NULL);

    quit = gtk_menu_item_new_with_label ("Quit");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "quit", quit);
    gtk_widget_show (quit);
    gtk_container_add (GTK_CONTAINER (file_menu), quit);
    gtk_signal_connect (GTK_OBJECT (quit), "activate",
			GTK_SIGNAL_FUNC (open_dialog),
			ConfirmQuitDialog);

    help = gtk_menu_item_new_with_label ("Help");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "help", help);
    gtk_widget_show (help);
    gtk_container_add (GTK_CONTAINER (menubar), help);
    gtk_menu_item_right_justify (GTK_MENU_ITEM (help));

    help_menu = gtk_menu_new ();
    gtk_object_set_data (GTK_OBJECT (SienaObject), "help_menu", help_menu);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (help), help_menu);

    about = gtk_menu_item_new_with_label ("About");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "about", about);
    gtk_widget_show (about);
    gtk_container_add (GTK_CONTAINER (help_menu), about);
    gtk_signal_connect (GTK_OBJECT (about), "activate",
			GTK_SIGNAL_FUNC (open_dialog),
			AboutDialog);

    request_frame = gtk_frame_new ("Siena Request");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "request_frame", request_frame);
    gtk_widget_show (request_frame);
    gtk_box_pack_start (GTK_BOX (vbox1), request_frame, FALSE, FALSE, 0);

    hbox2 = gtk_hbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "hbox2", hbox2);
    gtk_widget_show (hbox2);
    gtk_container_add (GTK_CONTAINER (request_frame), hbox2);

    SienaText = gtk_text_new (NULL, NULL);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "SienaText", SienaText);
    gtk_text_set_editable (GTK_TEXT (SienaText), TRUE);
    gtk_widget_show (SienaText);
    gtk_box_pack_start (GTK_BOX (hbox2), SienaText, TRUE, TRUE, 0);
    sty = gtk_widget_get_style(SienaText);
    sty->font = LogFont;
    gtk_widget_set_style(SienaText, sty);

    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "vbox2", vbox2);
    gtk_widget_show (vbox2);
    gtk_box_pack_start (GTK_BOX (hbox2), vbox2, TRUE, TRUE, 0);

    publish = gtk_button_new_with_label ("publish");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "publish", publish);
    gtk_widget_show (publish);
    gtk_box_pack_start (GTK_BOX (vbox2), publish, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (publish), "clicked",
			GTK_SIGNAL_FUNC (publish_clicked),
			SienaText);

    hbox3 = gtk_hbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "hbox3", hbox3);
    gtk_widget_show (hbox3);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox3, TRUE, TRUE, 0);

    subscribe = gtk_button_new_with_label ("subscribe");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "subscribe", subscribe);
    gtk_widget_show (subscribe);
    gtk_box_pack_start (GTK_BOX (hbox3), subscribe, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (subscribe), "clicked",
			GTK_SIGNAL_FUNC (subscribe_clicked), 
			(void *)&SENP::SUB);

    unsubscribe = gtk_button_new_with_label ("unsubscribe");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "unsubscribe", unsubscribe);
    gtk_widget_show (unsubscribe);
    gtk_box_pack_start (GTK_BOX (hbox3), unsubscribe, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (unsubscribe), "clicked",
			GTK_SIGNAL_FUNC (subscribe_clicked), 
			(void *)&SENP::UNS);

    hbox4 = gtk_hbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "hbox4", hbox4);
    gtk_widget_show (hbox4);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 0);

    advertise = gtk_button_new_with_label ("advertise");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "advertise", advertise);
    gtk_widget_show (advertise);
    gtk_box_pack_start (GTK_BOX (hbox4), advertise, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (advertise), "clicked",
			GTK_SIGNAL_FUNC (subscribe_clicked),
			(void *)&SENP::ADV);

    unadvertise = gtk_button_new_with_label ("unadvertise");
    gtk_object_set_data (GTK_OBJECT (SienaObject), "unadvertise", unadvertise);
    gtk_widget_show (unadvertise);
    gtk_box_pack_start (GTK_BOX (hbox4), unadvertise, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (unadvertise), "clicked",
			GTK_SIGNAL_FUNC (subscribe_clicked), 
			(void *)&SENP::UNA);

    hbox1 = gtk_hbox_new (FALSE, 0);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "hbox1", hbox1);
    gtk_widget_show (hbox1);
    gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

    LogText = gtk_text_new (NULL, NULL);
    gtk_object_set_data (GTK_OBJECT (SienaObject), "LogText", LogText);
    gtk_widget_show (LogText);
    gtk_box_pack_start (GTK_BOX (hbox1), LogText, TRUE, TRUE, 0);

    scrollbar = gtk_vscrollbar_new (GTK_ADJUSTMENT (GTK_TEXT(LogText)->vadj ));
    gtk_object_set_data (GTK_OBJECT (SienaObject), "scrollbar", scrollbar);
    gtk_widget_show (scrollbar);
    gtk_box_pack_start (GTK_BOX (hbox1), scrollbar, FALSE, TRUE, 0);

    StatusBar = gtk_statusbar_new ();
    gtk_object_set_data (GTK_OBJECT (SienaObject), "statusbar", StatusBar);
    gtk_widget_show (StatusBar);
    gtk_box_pack_start (GTK_BOX (vbox1), StatusBar, FALSE, TRUE, 0);
}

static void readMessage (gpointer data, gint source, 
			 GdkInputCondition condition)
{
    Event e;
    Request r;
    TCPSocket s;
    try {
	s = Port->accept();
	s >> r >> e;
	ostringstream ostr;
	ostr << e;
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&EventColor, &BackgroundColor,
			ostr.str().data(), ostr.str().length());
	gtk_text_insert(GTK_TEXT(LogText), LogFont,
			&EventColor, &BackgroundColor, "\n", 1);
	s << "200 notification accepted" << endl;
	s.close();
    } catch (exception &ex) {
	gtk_text_insert(GTK_TEXT(LogText), LogFont, 
			&ErrorColor, &BackgroundColor, 
			ex.what(), strlen(ex.what()));
	gtk_text_insert(GTK_TEXT(LogText), LogFont, &ErrorColor, &BackgroundColor,
			"\n", 1);
    }
};

void print_usage(const char *programname) 
{
    cerr << "Usage: " << programname << " <URI> [options...]\n\
options:\n\t-port number\n\t-monitor hostname\n\t-monitorport hostname\n\t-monitoricon iconname\n\t-nomonitor\n\t-host address\n\t-fn font" << endl;
    exit(-1);
}

int main (int argc, char *argv[])
{
    unsigned short port = 0;
    const char * monitor = "localhost";
    const char * monitoricon = "thinclient";
    unsigned short monitor_port = 1996;
    const char * portstr = NULL;
    const char * hname = NULL;

    portstr = getenv("MONITORPORT");
    monitor = getenv("MONITOR");

    if (argc < 2) {
	print_usage(argv[0]);
	exit(-1);
    }
    for (int i=2; i< argc; i++) {		// parses command-line parameters
	if (strcmp(argv[i], "-port")==0) {
	    if (++i < argc) {
		port = atoi(argv[i]);
	    } else {
		print_usage(argv[0]);
	    }
	} 
	else if (strcmp(argv[i], "-monitorport")==0) {
	    if (++i < argc) {
		portstr = argv[i];
	    } else {
		print_usage(argv[0]);
	    }
	} 
 	else if (strcmp(argv[i], "-monitor")==0) {
	    if (++i < argc) {
		monitor = argv[i];
	    } else {
		print_usage(argv[0]);
	    }
	} 
 	else if (strcmp(argv[i], "-fn")==0) {
	    if (++i < argc) {
		LogFontName = argv[i];
	    } else {
		print_usage(argv[0]);
	    }
	} 
 	else if (strcmp(argv[i], "-monitoricon")==0) {
	    if (++i < argc) {
		monitoricon = argv[i];
	    } else {
		print_usage(argv[0]);
	    }
	} 
 	else if (strcmp(argv[i], "-host")==0) {
	    if (++i < argc) {
		hname = argv[i];
	    } else {
		print_usage(argv[0]);
	    }
	} 
	else if (strcmp(argv[i], "-nomonitor")==0) {
	    monitor = NULL;
	} 
	else {
	    print_usage(argv[0]);
	}
    }
    if (portstr != NULL) monitor_port = atoi(portstr);

    try {
	Uri = new URI(argv[1]);
	UriStr = Uri->to_string();
	

	Port = new TCPPort(port);

	ostringstream ostr;
	ostr << "senp://";
	if (hname == NULL) {
	    char hostname[MAXHOSTNAMELEN];
	    gethostname(hostname, MAXHOSTNAMELEN -1);
	    INAddress addr(hostname, 0);
	    ostr << addr.hostname();
	} else {
	    ostr << hname; 
	}
	ostr << ":" << Port->local_address().port();
	MyUriStr = ostr.str();
	if (monitor != NULL) {
	    string mastername;
	    SienaMonitor = new SimpleMonitor(monitor, monitor_port);
	    SienaMonitor->add_node(MyUriStr, monitoricon);
	    SienaMonitor->connect(MyUriStr, UriStr);
	}

	string sfrom(ostr.str());
	PubRequest[SENP::Method] = SENP::PUB;
	PubRequest[SENP::From] = sfrom;
	SubRequest[SENP::Subscriber] = sfrom;
	SubRequest[SENP::From] = sfrom;

	cout << "My address: " << sfrom << endl;

	gtk_set_locale();
	gtk_init(&argc, &argv);
    
	init_fonts();
	init_colors();	

	create_AboutDialog();
	create_ConfirmQuitDialog();
    
	create_SienaObject(argv[1]);

	int ContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), 
						     "Statusbar");
	gtk_statusbar_push(GTK_STATUSBAR(StatusBar), ContextId, 
			   sfrom.c_str());
	gtk_widget_show(SienaObject);
	gdk_input_add(Port->filedescr(), GDK_INPUT_READ, readMessage, NULL);
	gtk_main();

	delete(Uri);
	return 0;
    } catch (exception &ex) {
	cerr << ex.what() << endl;
	exit(-1);
    }
}
